#!/usr/bin/env python
# -*- coding:utf-8 -*-

from lcyframe import route
from lcyframe import funts
from base import BaseHandler, helper

@route("/permission/groups")
class PermissionGroupsHandler(BaseHandler):
    
    @helper.admin(1)
    def get(self):
        """
        角色详情
        
        Request extra query params:
        - group_id # type: int required:True 角色id
        
        :return:
        :rtype:
        """

        group_mp = self.model.GroupsModel.get_groups_mapping(self.params["content_type"])
        if not group_mp or str(self.params["group_id"]) not in group_mp:
            raise self.api_error.ErrorCommonPermission
        group = group_mp[str(self.params["group_id"])]
        group = self.model.GroupsModel.get_members_count(None, group)[0]
        self.write_success(group)


    @helper.admin(1)
    def post(self):
        """
        新增角色

        Request extra query params:
        - name # type: str required:True 权限名称
        - content_type # type: int required:True 0系统端角色，1业务的角色
        - describes # type: str required:False 角色说明
        
        :return:
        :rtype:
        """
        name = self.params.pop("name")
        content_type = self.params.pop("content_type")
        self.model.GroupsModel.create_groups(name, content_type, **self.params)
        self.write_success()

    @helper.admin(1)
    def put(self):
        """
        修改角色

        Request extra query params:
        - group_id # type: int required:True id
        - name # type: str required:False 权限名称
        - state # type: int required:False 1启用，0禁用
        - describes # type: str required:False 角色说明
        
        :return:
        :rtype:
        """
        group_id = self.params.pop("group_id")
        content_type = self.params.pop("content_type")
        self.model.GroupsModel.update_group(group_id, content_type, **self.params)
        self.write_success()

    @helper.admin(1)
    def delete(self):
        """
        删除角色
        
        Request extra query params:
        - group_id # type: int required:True id
        
        :return:
        :rtype:
        """
        
        # self.model.GroupsModel.delete(self.params["oid"])
        self.write_success()


@route("/permission/groups/list")
class PermissionGroupsListHandler(BaseHandler):
    
    @helper.admin(1)
    def get(self):
        """
        角色列表
        
        Request extra query params:
        - page # type: int required:False 翻页码
        - count # type: int required:False 每页显示条数
        - state # type: int required:False 状态
        - content_type # type: int required:True 0系统端角色，1业务的角色
        - company_id # type: int required:True 指定商户，默认所有角色。
        
        :return:
        :rtype:
        """

        count = self.params.get("count", 10)
        page = self.params.get("page", 1)

        sql_and = {}
        orderby = {
            "id": -1
        }
        if self.params.get("content_type"):
            sql_and["content_type"] = self.params["content_type"]
        if self.params.get("state"):
            sql_and["state"] = self.params["state"]
        if self.params.get("company_id"):
            sql_and["company_id"] = self.params["company_id"]

        datas, counts = self.model.GroupsModel.get_group_list_by_page(
            page, count,
            sql_and=sql_and,
            orderby=orderby)

        datas = self.model.GroupsModel.get_members_count(self.params.get("company_id"), *datas)

        self.write_pagination(datas, counts)


@route("/permission/groups/members")
class PermissionGroupsMembersHandler(BaseHandler):

    @helper.admin(1)
    def get(self):
        """
        角色的成员
        查看指定角色或指定商户的成员列表
        :return:
        :rtype:
        """
        company_id = self.params.pop("company_id", None)
        group = self.model.GroupsModel.get_data_by_kv(id=self.params["group_id"])
        datas = self.model.GroupsModel.get_members(company_id, group)
        company_mp = self.model.CompanyModel.get_batch_by_in("name", id=[data["company_id"] for data in datas])
        for data in datas:
            data["company_name"] = company_mp[data["company_id"]]["name"]
        self.write_success({"datas": datas})

    @helper.admin(1)
    def post(self):
        """
        该角色分配成员
        """
        user_ids = list(map(int, self.params["user_id"].split(",")))
        company_id = self.params.pop("company_id")
        group_id = self.params.pop("group_id")
        group = self.model.GroupsModel.get_data_by_kv(id=group_id)
        if not group:
            raise self.api_error.ErrorCommonPermission("角色不存在")

        user_mp = self.model.AdminModel.get_batch_by_in("id", "group_id", "company_id", id=user_ids)
        for id, user in user_mp.items():
            if user["company_id"] != company_id:
                raise self.api_error.ErrorCommonPermission("用户%s不属于该商户"%user["username"])
            group_ids = set(user["group_id"].split(","))
            group_ids.add(str(group_id))
            group_ids = group_ids-set([0, "0"])
            self.model.AdminModel.update_data(user["id"], group_id=",".join(list(group_ids)))
        self.write_success()

@route("/permission/groups/check")
class PermissionGroupsCheckHandler(BaseHandler):

    @helper.admin(1)
    def post(self):
        """
        编辑角色权限
        权限勾选或取消,修改成功后，建议刷新本地权限缓存
        
        Request extra query params:
        - group_id # type: int required:True 角色id
        - key # type: str required:True 接口标识符
        - method # type: str required:True 接口方法
        - value # type: int required:True 1勾选，0取消

        :return:
        :rtype:
        """

        group_id = self.params["group_id"]
        key = self.params["key"]
        method = self.params["method"]
        value = self.params["value":]

        self.model.GroupsModel.update_group_permission(group_id, key, method, value)
        self.write_success()


@route("/permission/groups/checks")
class PermissionGroupsCheckSHandler(BaseHandler):

    @helper.admin(1)
    def post(self):
        """
        批量修改
        编辑角色权限
        权限勾选或取消,修改成功后，建议刷新本地权限缓存

        Request extra query params:
        - group_id # type: int required:True 角色id

        :return:
        :rtype:
        """

        group_id = self.params["group_id"]
        permissions = self.params["permissions"]
        self.model.GroupsModel.update_group_permissions(group_id, permissions)
        self.write_success()

@route("/permission/groups/company")
class PermissionGroupsCompanyHandler(BaseHandler):
    
    @helper.admin(1)
    def get(self):
        """
        商户的所有角色
        查看某商户的所有角色，不分页
        
        Request extra query params:
        - company_id # type: int required:True 商户id
        
        :return:
        :rtype:
        """
        content_type = self.params["content_type"]
        if content_type:
            if not self.params["company_id"]:
                raise self.api_error.ErrorCommonPermission("需提供商户id")
            data = self.model.CompanyGroupsModel.get_company_groups(content_type)
            groups = data[str(self.params["company_id"])]
        else:
            groups = self.model.CompanyGroupsModel.get_system_groups()

        groups_mp = self.model.GroupsModel.get_groups_mapping(content_type)

        for index, group_id in enumerate(groups):
            groups[index] = groups_mp[str(group_id)]

        groups = self.model.GroupsModel.get_members_count(self.params["company_id"], *groups)

        self.write_success(data={"datas": groups})
            

    @helper.admin(1)
    def post(self):
        """
        分配角色
        把角色分配给商户
        
        Request extra query params:
        - group_id # type: str required:True 角色id， 多个角色用英文逗号隔开
        - company_id # type: int required:True 商户id
        
        :return:
        :rtype:
        """
        group_id = self.params.pop("group_id", "").split(",")
        company_id = self.params.pop("company_id")
        self.model.CompanyGroupsModel.update_company_group(company_id, group_id)
        self.write_success()

    @helper.admin(1)
    def delete(self):
        """
        移除角色
        将角色从商户下移除，属于该角色的用户将失去该角色的所有权限
        
        Request extra query params:
        - group_id # type: int required:True 角色id
        - company_id # type: int required:True 商户id

        :return:
        :rtype:
        """

        group_id = self.params.pop("group_id", "").split(",")
        company_id = self.params.pop("company_id")
        self.model.CompanyGroupsModel.remove_company_group(company_id, group_id)
        self.write_success()


@route("/permission")
class PermissionHandler(BaseHandler):
    
    @helper.admin(1)
    def get(self):
        """
        权限详情
        
        Request extra query params:
        - permission_id # type: int required:True id
        
        :return:
        :rtype:
        """

        data = self.model.RBACPermissionModel.get_data_by_kv(id=self.params["permission_id"])

        self.write_success(data)
            

    @helper.admin(1)
    def post(self):
        """
        新增

        Request extra query params:
        - content_type # type: int required:True 0系统端角色，1业务的角色
        - key # type: str required:True 接口标识符
        - name # type: str required:True 名称
        - permission_class # type: int required:True 类别，tag标签，api接口权限
        - method # type: str required:False 接口方法， 当permission_class=api时必须提供
        - parent_id # type: str required:False 父级id
        - describes # type: str required:False 角色说明
        - state # type: int required:False 角色说明
        - hook_func # type: str required:False 钩子方法
        
        :return:
        :rtype:
        """
        key = self.params.pop("key")
        name = self.params.pop("name")
        permission_class = self.params.pop("permission_class")
        content_type = self.params.pop("content_type")
        self.model.RBACPermissionModel.create_permission(key, name, permission_class, content_type, **self.params)
        self.write_success()

    @helper.admin(1)
    def put(self):
        """
        修改
        修改权限名称，描述，状态，钩子
        
        Request extra query params:
        - content_type # type: int required:True 0系统端角色，1业务的角色
        - key # type: str required:False 接口标识符
        - name # type: str required:False 名称
        - permission_class # type: int required:False 类别，tag标签，api接口权限
        - method # type: str required:False 接口方法， 当permission_class=api时必须提供
        - parent_id # type: str required:False 父级id
        - describes # type: str required:False 角色说明
        - state # type: int required:False 角色说明
        - hook_func # type: str required:False 钩子方法
        
        :return:
        :rtype:
        """
        
        permission_id = self.params.pop("permission_id")
        content_type = self.params.pop("content_type")
        self.model.RBACPermissionModel.update_permission(permission_id, content_type, **self.params)
        self.write_success()

    @helper.admin(1)
    def delete(self):
        """
        删除

        Request extra query params:
        - permission_id # type: int required:True id

        :return:
        :rtype:
        """
        permission_id = self.params.pop("permission_id")
        content_type = self.params.pop("content_type")
        self.model.RBACPermissionModel.remove_permission(permission_id, content_type)
        self.write_success()
        

@route("/permission/list")
class PermissionListHandler(BaseHandler):
    
    @helper.admin(1)
    def get(self):
        """
        权限条目清单
        
        Request extra query params:
        - state # type: int required:False 状态
        - content_type # type: int required:True 0系统端权限条目，1业务端权限条目
        
        :return:
        :rtype:
        """
        permissions = []
        for item in self.model.RBACPermissionModel.get_permissions_list(self.params["content_type"]):
            if item["state"] == 1:
                permissions.append(item)

        self.write_success({"datas": permissions})

@route("/permission/enable")
class PermissionEnableHandler(BaseHandler):
    
    @helper.admin(1)
    def get(self):
        """
        是否启用权限系统
        
        :return:
        :rtype:
        """
        data = self.model.RBACPermissionModel.is_enable(self.params["content_type"])
        self.write_success(data={
            "enable": 1 if data else 0
        })
            

    @helper.admin(1)
    def post(self):
        """
        启用停用
        停用权限系统后，所有访问都不做拦截
        
        Request extra query params:
        - state # type: int required:True 状态， 1启用，0停用

        :return:
        :rtype:
        """
        state = self.params.pop("state")
        if state:
            self.model.RBACPermissionModel.enable_permission(self.params["content_type"])
        else:
            self.model.RBACPermissionModel.disable_permission(self.params["content_type"])
        self.write_success()


@route("/permission/loads")
class PermissionLoadseHandler(BaseHandler):

    @helper.admin(1)
    def post(self):
        """
        导入新的权限条目
        当后端新增新的api逻辑后，调用该方法可自动载入新增的权限条目。该功能仅支持系统端
        Request extra query params:
        - state # type: int required:True 状态， 1启用，0停用

        :return:
        :rtype:
        """
        new_permission = {}

        routes = {}
        routes_hander_mp = {}
        exists_permission = {}
        for item in self.application.handlers:
            key = item.name
            if key in ["DocsHandler", "BaseHandler"]:
                continue
            routes_hander_mp[key] = item.handler_class
            funcs = vars(item.target).keys()
            routes.setdefault(key, [])
            for method in ["get", "post", "put", "delete"]:
                if method in funcs:
                    routes[key].append(method)
                else:
                    continue

        # 已存在的权限
        permissions = self.model.RBACPermissionModel.get_datas_by_spec(sql_and={"content_type": 0})
        for permission in permissions:
            exists_permission[permission["key"]] = permission["method"]

        # 筛选出未导入的选线条目
        for key, methods in routes.items():
            if not methods:
                continue
            if key not in exists_permission:
                new_permission[key] = methods
            else:
                for method in exists_permission[key].split(","):
                    if key in new_permission and method in new_permission:
                        new_permission[key].remove(method)

        datas = self.model.RBACPermissionModel.autoload_permission(routes_hander_mp, new_permission)
        self.write_success({"datas": datas})


import os, sys, json, time
from functools import wraps





class RBACPermission(object):
    """
    权限拦截尽量不入侵到model层，哪怕是逻辑权限也尽可能在handler层处理
    """

    def __init__(self, handler):
        self.handler = handler
        # self.model = handler.model

    def has_permission(self, request):
        '''
        根据用户权限条目判断当前用户是否有权限请求
        :param request:
        :param permission_dict:
        :return:
        '''
        msg = None
        has_perm = False

        resolve_url_obj = request.path
        current_url_name = resolve_url_obj.url_name
        current_method = request.method.lower()

        # # 企业账号免校验，拥有所有权限
        # if request.user.is_superuser:
        #     return True, msg

        # 是否启用权限系统
        if self.is_enable(1) != True:
            return True, msg

        groups_permission = self.get_groups_mapping()
        # 支持用户有多个角色，只要其中一个角色有权限，即代表有权限
        for group_id in str(request.user.group_id).split(','):
            if str(group_id) not in groups_permission:
                continue

            permission_dict = groups_permission[str(group_id)]
            if permission_dict["state"]:         # 角色禁用，视为无权限
                key = current_url_name + ":" + current_method
                val = permission_dict["permissions"].get(key)
                if val:               # 1有权限，0无权限, None
                    has_perm = True
                    permission_mapping = self.get_permissions_mapping()
                    if key in permission_mapping:
                        permission = permission_mapping[key]
                        hook_func = permission["hook_func"]
                        if hook_func and hasattr(permission_hook, hook_func):
                            try:
                                results = getattr(permission_hook, hook_func)(request)
                                if isinstance(results, tuple) and len(results) == 2:
                                    flag, msg = results
                                else:
                                    flag, msg = results, ""
                            except Exception as e:
                                print(str(e))
                            else:
                                if flag:
                                    has_perm = True
                    if has_perm:
                        break
                    else:
                        continue

        return has_perm, msg


    def check_permission(self, func):
        @wraps(func)
        def inner(*args, **kwargs):
            has_per, msg = self.has_permission(*args, **kwargs)
            if not has_per:
                return JsonResponse({
                    'code': 403,
                    'msg': '没有权限'
                })
            return func(*args, **kwargs)

        return inner

    def rmcache(self, func):
        @wraps(func)
        def remove(*args, **kwargs):
            self.remove_redis()
            self.set_version()
            return func(*args, **kwargs)

        return remove

    def remove_redis(self):
        content_type = 1
        redis.delete("groups:%s" % content_type)
        redis.delete("permissions:%s" % content_type)
        redis.delete("company_groups:%s" % content_type)

    def set_version(self):
        """
        每一次修改权限，版本号为当前时间戳
        :param content_type:
        :return:
        """
        content_type = 1
        redis.set("permission_version:%s" % content_type, int(time.time()))

    def get_version(self, content_type=1):
        """
        获取当前版本号
        :param content_type:
        :return:
        """
        current_version = redis.get("permission_version:%s" % content_type) or 0
        return int(current_version)

    def has_update(self, client_version, content_type=1):
        """
        客户端是否需要更新权限
        当客户端请求的header.permission_version<当前版本号时，
        设置响应header.permission_version=current_version
        客户端判断响应头里包含header.permission_version时，重新请求权限数据，并刷新客户端权限配置
        :param client_version:
        :param content_type:
        :return:
        """
        return self.get_version(content_type) > client_version

    def create_groups(self, name, content_type, **group):
        """
        创建角色
        name = models.CharField(max_length=128)
        state = models.IntegerField(default=1)
        desc = models.CharField(max_length=128, default="", verbose_name='描述')
        permissions = models.JSONField(default=dict)
        content_type = models.IntegerField(default=1, verbose_name='1业务角色，2管理端角色')
        assign = models.IntegerField(default=1, verbose_name='是否默认分配给商户,1是0否')
        :return:
        """

        if content_type not in [0, 1]:
            raise Exception("参数错误")
        if Groups.objects.filter(name=name, content_type=content_type):
            raise Exception("已存在同名角色")
        group["name"] = name
        group["content_type"] = content_type
        return Groups.objects.create(**group)

    @rmcache
    def update_group(self, group_id, **kwargs):
        """
        更新角色信息: 名称 描述 状态 分配
        :param id:
        :param kwargs:
        :return:
        """
        kwargs.pop("id", "")
        kwargs.pop("permissions", "")
        kwargs.pop("content_type", "")
        if kwargs:
            Groups.objects.filter(id=group_id).update(**kwargs)

    @rmcache
    def update_user_group(self, *uids, group_id=None):
        """
        修改用户的角色
        :param uids:
        :param group_id: "gid1,gid2,gid3"
        :return:
        """
        try:
            user = User.objects.get(id=uids[0])
            company = Company.objects.get(id=user.company_id)
        except Exception as e:
            return
        if not CompanyGroups.objects.filter(company_id=company.id, group_id=group_id):
            return
        User.objects.filter(id__in=uids).update(group_id=group_id)

    @rmcache
    def update_company_group(self, company_id, group_ids):
        """
        修改、新增商户的角色
        :param old_group:
        :param new_group:
        :return:
        """
        group_ids = [group_ids] if not isinstance(group_ids, (list, tuple)) else group_ids
        for group_id in group_ids:
            CompanyGroups.objects.update_or_create(company_id=company_id, group_id=group_id)

    @rmcache
    def remove_company_group(self, company_id, group_ids):
        """
        移除商户的某个角色：所有已分配该角色的用户，将被解除角色绑定
        :param company_id:
        :param group_id:
        :return:
        """
        group_ids = [str(i) for i in ([group_ids] if not isinstance(group_ids, (list, tuple)) else group_ids)]
        # 删除角色
        for group_id in group_ids:
            try:
                data = CompanyGroups.objects.get(company_id=company_id, group_id=group_id)
                data.delete()
            except Exception as e:
                pass

        # 从该用户角色中剔除这些角色
        for user in User.objects.filter(company_id=company_id):
            user_group_ids = set(user.group_id.split(","))
            group_ids = set(group_ids)
            user.group_id = ",".join(user_group_ids.difference_update(group_ids))
            user.save()

    @rmcache
    def update_group_permission(self, group_id, key, method, val):
        """
        勾选、取消权限
        :param group_id:
        :param key:
        :param method:
        :return:
        """
        group = Groups.objects.get(id=group_id)
        if group:
            permissions = group.permissions
            for method in method.split(","):
                if val:
                    permissions[key+":"+method] = val
                else:
                    permissions.pop(key+":"+method, "")
            group.save()

    @rmcache
    def create_permission(self, key, name, permission_class, content_type, **kwargs):
        """
        新增权限条目
        :param id:
        :param kwargs:
        :return:
        """
        if content_type not in [0, 1]:
            raise Exception("参数错误")
        if Permissions.objects.filter(key=key):
            raise Exception("<%s>已被使用" % key)
        if Permissions.objects.filter(name=name, content_type=content_type):
            raise Exception("<%s>已被使用" % name)
        if kwargs.get("hook_func") and Permissions.objects.filter(hook_func=kwargs.get("hook_func")):
            raise Exception("<%s>已存在" % kwargs.get("hook_func"))
        if permission_class not in ["tag", "api", "logic逻辑权限"]:
            raise Exception("permission_class只能是tag，api，logic中的一个")
        if permission_class != "tag":
            if not kwargs.get("method"):
                raise Exception("没有提供方法名method")
            for method in kwargs.get("method").split(","):
                if method not in ["get", "delete", "put", "post"]:
                    raise Exception("不支持该方法名" % method)
        kwargs["name"] = name
        kwargs["key"] = key
        kwargs["permission_class"] = permission_class
        kwargs["content_type"] = int(content_type)
        kwargs.setdefault("state", 1)
        return Permissions.objects.create(**kwargs)

    @rmcache
    def update_permission(self, permission_id, **kwargs):
        """
        更新权限信息: 名称 描述 状态 钩子 父级
        :param id:
        :param kwargs:
        :return:
        """
        kwargs.pop("id", "")
        kwargs.pop("key", "")
        kwargs.pop("method", "")
        kwargs.pop("permission_class", "")
        kwargs.pop("content_type", "")
        if kwargs:
            Permissions.objects.filter(id=permission_id).update(**kwargs)


    @rmcache
    def remove_permission(self, permission_id):
        """
        删除权限条目
        :param id:
        :param kwargs:
        :return:
        """
        return Permissions.objects.filter(id=permission_id).delete()

    def get_permissions_list(self, content_type=1):
        """
        前端
        权限条目
        导出为json,该数据结构返回给前端
        :return:
        """
        permissions = redis.get("permissions:%s" % content_type)
        if permissions:
            permissions = json.loads(permissions)
        else:
            permissions = []     # 顶级
            all_mapping = {}     # 所有数据的key：value
            datas = list(Permissions.objects.filter(state=1, content_type=content_type))   # 按id正序读取
            for p in datas:
                p = model_to_dict(p)
                if not p["parent_id"]:
                    permissions.append(p)
                all_mapping[p["id"]] = p
                if p["parent_id"]:
                    all_mapping[p["parent_id"]].setdefault("children", [])
                    all_mapping[p["parent_id"]]["children"].append(p)

            if permissions:
                redis.set("permissions:%s" % content_type, json.dumps(permissions))
        return permissions

    def get_permissions_mapping(self, content_type=1):
        """
        权限是否启用：1是，0禁用
        :return:
        {
            key:method: 1 or 0, # methods含多个方法时，平铺为多个key和单方法，例如key:method
            key:get: 1,         # 键：权限key:方法(单个方法)
            key:post: 1,        # 键：权限key:方法(单个方法)
        }
        """
        permissions = self.get_permissions_list(content_type)
        permissions_mapping = {}
        def collect_api_permission(item, mapping):
            if item.get("children"):
                childrens = item.pop("children")
                for children in childrens:
                    collect_api_permission(children, mapping)
            else:
                # 权限禁用后，不再进行权限校验等同于有权访问
                if item["permission_class"] == "api" and item["state"]:
                    for method in item["method"].split(","):
                        mapping[item["key"] + ":" + method] = item
                else:
                    return mapping

        for p in permissions:
            collect_api_permission(p, permissions_mapping)
        return permissions_mapping

    def get_groups_list(self, content_type=1):
        """
        前端
        返回角色（含权限）
        :return:
        [
            {
                "id": "id",
                "name": "角色名称",
                # "group_class": "技术部",             # 预留，自定义标签
                "state": 1,                           # 1可以 0 禁用 默认1
                "content_type": 1 or 0,             # 1 业务端使用，0系统管理端
                "assign": 0,                        # 是否默认分配给商户 1是0否
                "permissions": {
                    "control2-xxx-api:get": 1,        # 1勾选，0未勾选
                    "control2-xxx-api2:get,post,put,delete": 1,  # 组合方法
                    "monitoring-tasks-api:post": 1
                }
            },
            ...
        ]
        """
        groups = redis.get("groups:%s" % content_type)
        if groups:
            groups = json.loads(groups)
        else:
            groups = []
            for g in Groups.objects.filter(state=1, content_type=content_type):
                groups.append(model_to_dict(g))
            if groups:
                redis.set("groups:%s" % content_type, json.dumps(groups))
        return groups

    def get_groups_mapping(self, content_type=1):
        """
        获取角色拥有的权限，多个方法组合为一个权限时，会得到多条k：v
        :return:
        {
            1: {
                "...": **groups,                  # 角色基础信息
                "permissions": {
                    groups.key:get: 1,         # 键：权限key:方法(单个方法)
                    groups.key:post: 1,        # 键：权限key:方法(单个方法)
                    ...
                }
            }
        }
        """
        groups = {str(g["id"]): g for g in self.get_groups_list(content_type)}
        for id, group in groups.items():
            temp = {}
            permissions = group.pop("permissions")
            # get,post 多个方法平铺
            for keys, val in permissions.items():
                if not val:
                    continue  # 0代表没有权限，跳过
                key, methods = keys.split(":")
                if "," in methods:
                    for method in methods.split(","):
                        temp[key + ":" + method] = val
                else:
                    temp[keys] = val
            group["permissions"] = temp
        return groups

    def get_system_groups(self):
        """
        获取系统角色
        :return:
        """
        return self.get_company_groups(0)

    def get_company_groups(self, content_type=1):
        """
        商户对应的角色
        :param content_type:
        :return:
        """
        company_groups = {}
        cache_datas = redis.get("company_groups:%s" % content_type)
        if cache_datas:
            company_groups = json.loads(cache_datas)
        else:
            company_groups[str(content_type)] = []
            if content_type == 1:
                datas = CompanyGroups.objects.all()
                for item in datas:
                    if str(item.company_id) not in company_groups:
                        company_groups[str(item.company_id)] = []
                    company_groups[str(item.company_id)].append(item.group_id)
            else:
                for group in Groups.objects.filter(content_type=content_type):
                    company_groups[str(content_type)].append(group.id)

            if company_groups:
                redis.set("company_groups:%s" % content_type, json.dumps(company_groups))

        return company_groups

    def enable_permission(self, content_type=1):
        """
        启用权限系统
        :param content_type:
        :return:
        """
        redis.set("permission_enable:%s" % content_type, 1)

    def disable_permission(self, content_type=1):
        """
        停用权限系统
        :param content_type:
        :return:
        """
        redis.set("permission_enable:%s" % content_type, 0)

    def is_enable(self, content_type=1):
        return redis.get("permission_enable:%s" % content_type) == b'1'

if __name__ == '__main__':
    # create_groups(**{"name": "新角色"})
    # update_group(19, name="单位主账号", desc="dSsd")
    # create_permission("xxx-key", "xxx-name", "api", 1, method="get")
    # remove_permission(274)
    # update_permission(154, desc="修改钩子", hook_func="func1,func2", parent_id=145)
    # # update_company_group(2, [5,8])
    # # update_user_group(58, group_id=5)
    # # remove_company_group(2, 5)
    # # update_group_permission(5, "user_detail", "get", 1)
    #
    # print(get_permissions_list(1))
    # print(get_permissions_mapping(1))
    # print(get_groups_list(1))
    print(get_company_groups(1))
    # print(get_system_groups())
    # enable_permission()
    # disable_permission()
    # print(is_enable())

    # 版本
    # print(get_version(1))
    # print(has_update(0, 1))
    pass


import logging
from datetime import datetime

from contextlib import suppress
from django.conf import settings
from django.urls import resolve
from django.db.models import Q
from mongoengine.queryset.visitor import Q as MONGO_Q

# from .publick_tools import get_ur_subdomain

logger = logging.getLogger("django")


def check_permission_hook(request):
    print("check_permission_hook")
    return True


# def tasks_tasks_post(request):
#     """
#     扫描任务判断
#     ------判断user的公司的权限组
#     :param request:
#     :return:
#     """
#     msg = "ok"
#
#     try:
#         if Tasks.objects.filter(
#                 company_id=request.user.company_id,
#                 create_time__year=datetime.now().year,
#                 create_time__month=datetime.now().month
#         ).exclude(state=Tasks.STATE_ABEND).count() >= request.user.company.groups.role.scan_times:
#             msg = "扫描任务次数达到上限，请联系管理员"
#             return False, msg
#     except Exception as e:
#         msg = "用户异常，请联系客服"
#         return False, msg
#
#     return True, msg


def user_company_user_get(request):
    msg = "ok"
    # 接口限制, 数据限制
    # role_list = request.user.groups.all().values_list("id", flat=True)
    # if settings.ADMIN_ROLE_GROUP not in role_list:
    #     q = Q()
    #     q.connector = 'AND'
    #     q.children.append(('id', request.user.id))
    #     setattr(request, 'permission_qobject', q)
    return True, msg


def user_logging_register_get(request):
    msg = "ok"
    # 接口限制, 数据限制
    role_list = request.user.groups.all().values_list("id", flat=True)
    if settings.ADMIN_ROLE_GROUP not in role_list:
        q = MONGO_Q()
        q.AND = 1
        q.OR = 0
        q.query.setdefault("user_id", request.user.id)
        setattr(request, "permission_qobject", q)
    return True, msg


def user_logging_register_execl_get(request):
    msg = "ok"
    # 接口限制, 数据限制
    role_list = request.user.groups.all().values_list("id", flat=True)
    if settings.ADMIN_ROLE_GROUP not in role_list:
        q = MONGO_Q()
        q.AND = 1
        q.OR = 0
        q.query.setdefault("user_id", request.user.id)
        setattr(request, "permission_qobject", q)
    return True, msg


def user_logging_operation_get(request):
    msg = "ok"
    # 接口限制, 数据限制
    role_list = request.user.groups.all().values_list("id", flat=True)
    if settings.ADMIN_ROLE_GROUP not in role_list:
        q = MONGO_Q()
        q.AND = 1
        q.OR = 0
        q.query.setdefault("user_id", request.user.id)
        setattr(request, "permission_qobject", q)
    return True, msg


def user_logging_operation_execl_get(request):
    msg = "ok"
    # 接口限制, 数据限制
    role_list = request.user.groups.all().values_list("id", flat=True)
    if settings.ADMIN_ROLE_GROUP not in role_list:
        q = MONGO_Q()
        q.AND = 1
        q.OR = 0
        q.query.setdefault("user_id", request.user.id)
        setattr(request, "permission_qobject", q)
    return True, msg



def user_user_site_info_get(request):

    return True, ""





#!/usr/bin/env python
# -*- coding:utf-8 -*-
from base import BaseModel
from model.schema.permission_schema import GroupsSchema, CompanyGroupsSchema, PermissionsSchema
from utils.rbac import permission_hook


class RBACPermissionModel(BaseModel, PermissionsSchema):

    @classmethod
    def _parse_data(cls, doc, **kwargs):
        """
        组装、修正单条记录
        该方法若存在，框架会自动使用
        若不存在，则模式使用框架内置方法
        :param doc: 单条数据
        :param kwargs:
        :return:
        """
        if not doc:
            return {}

        if "parent_id" in doc and doc["parent_id"] == None:
            doc["parent_id"] = 0
        doc = cls.format_datetime(doc)

        return doc

    @classmethod
    def has_permission(cls, handler):
        '''
        根据用户权限条目判断当前用户是否有权限请求
        :param request:
        :param permission_dict:
        :return:
        '''
        msg = None
        has_perm = False

        current_path = handler.request.path
        current_url_name = handler.__class__.__name__
        current_method = handler.request.method.lower()

        # # 企业账号免校验，拥有所有权限
        # if request.user.is_superuser:
        #     return True, msg

        # 是否启用权限系统
        if cls.is_enable(0) != True:
            return True, msg

        groups_permission = cls.model.GroupsModel.get_groups_mapping(0)
        # 支持用户有多个角色，只要其中一个角色有权限，即代表有权限
        for group_id in str(handler.member["group_id"]).split(','):
            if str(group_id) not in groups_permission:
                continue

            permission_dict = groups_permission[str(group_id)]
            if permission_dict["state"]:  # 角色禁用，视为无权限
                key = current_url_name + ":" + current_method
                val = permission_dict["permissions"].get(key)
                if val:  # 1有权限，0无权限, None
                    has_perm = True
                    permission_mapping = cls.get_permissions_mapping(0)
                    if key in permission_mapping:
                        permission = permission_mapping[key]
                        hook_func = permission["hook_func"]
                        if hook_func and hasattr(permission_hook, hook_func):
                            try:
                                results = getattr(permission_hook, hook_func)(handler.request)
                                if isinstance(results, tuple) and len(results) == 2:
                                    flag, msg = results
                                else:
                                    flag, msg = results, ""
                            except Exception as e:
                                print(str(e))
                            else:
                                if flag:
                                    has_perm = True
                    if has_perm:
                        break
                    else:
                        continue

        return has_perm, msg

    @classmethod
    def create_permission(cls, key, name, permission_class, content_type, **kwargs):
        """
        新增权限条目
        :param id:
        :param kwargs:
        :return:
        """
        if content_type not in [0, 1]:
            raise Exception("参数错误")
        if cls.get_datas_by_spec(sql_and=dict(key=key)):
            raise Exception("`%s`已被使用" % key)
        if cls.get_datas_by_spec(sql_and=dict(name=name, content_type=content_type)):
            raise Exception("`%s`已被使用" % name)
        if kwargs.get("hook_func") and cls.get_datas_by_spec(sql_and=dict(hook_func=kwargs.get("hook_func"))):
            raise Exception("`%s`已存在" % kwargs.get("hook_func"))
        if permission_class not in ["tag", "api", "logic逻辑权限"]:
            raise Exception("permission_class只能是tag，api，logic中的一个")
        if permission_class != "tag":
            if not kwargs.get("method"):
                raise Exception("没有提供方法名method")
            for method in kwargs.get("method").split(","):
                if method not in ["get", "delete", "put", "post"]:
                    raise Exception("不支持该方法名" % method)
        kwargs["name"] = name
        kwargs["key"] = key
        kwargs["permission_class"] = permission_class
        kwargs["content_type"] = int(content_type)
        kwargs.setdefault("state", 1)

        cls.rmcache(content_type)
        return cls.create_data(**kwargs)

    @classmethod
    def update_permission(cls, permission_id, content_type, **kwargs):
        """
        单条更新权限信息: 名称 描述 状态（启用停用） 钩子 父级
        :param id:
        :param kwargs:
        :return:
        """
        kwargs.pop("id", "")
        kwargs.pop("key", "")
        # kwargs.pop("method", "")
        kwargs.pop("permission_class", "")
        if kwargs:
            cls.rmcache(content_type)
            cls.update_data(permission_id, **kwargs)

    @classmethod
    def remove_permission(cls, permission_id, content_type):
        """
        删除权限条目:物理删除
        :param id:
        :param kwargs:
        :return:
        """
        cls.rmcache(content_type)
        return cls.mysql.execute(f"delete from {cls.collection} where id=%s", [permission_id])

    @classmethod
    def get_permissions_list(cls, content_type=1):
        """
        前端
        权限条目
        导出为json,该数据结构返回给前端
        :return:
        """
        permissions = cls.redis.get("permissions:%s" % content_type)
        if permissions:
            permissions = cls.utils.to_python(permissions)
        else:
            permissions = []  # 顶级
            all_mapping = {}  # 所有数据的key：value
            datas = list(cls.get_datas_by_spec(sql_and=dict(state=1, content_type=content_type)))  # 按id正序读取
            for p in datas:
                if not p["parent_id"]:
                    p["parent_id"] = 0
                    permissions.append(p)
                all_mapping[p["id"]] = p
                if p["parent_id"]:
                    all_mapping[p["parent_id"]].setdefault("children", [])
                    all_mapping[p["parent_id"]]["children"].append(p)

            if permissions:
                cls.redis.set("permissions:%s" % content_type, cls.utils.to_json(permissions))
        return permissions

    @classmethod
    def get_permissions_mapping(cls, content_type=1):
        """
        权限是否启用：1是，0禁用
        :return:
        {
            key:method: 1 or 0, # methods含多个方法时，平铺为多个key和单方法，例如key:method
            key:get: 1,         # 键：权限key:方法(单个方法)
            key:post: 1,        # 键：权限key:方法(单个方法)
        }
        """
        permissions = cls.get_permissions_list(content_type)
        permissions_mapping = {}

        def collect_api_permission(item, mapping):
            if item.get("children"):
                childrens = item.pop("children")
                for children in childrens:
                    collect_api_permission(children, mapping)
            else:
                # 权限禁用后，不再进行权限校验等同于有权访问
                if item["permission_class"] == "api" and item["state"]:
                    for method in item["method"].split(","):
                        mapping[item["key"] + ":" + method] = item
                else:
                    return mapping

        for p in permissions:
            collect_api_permission(p, permissions_mapping)
        return permissions_mapping

    @classmethod
    def enable_permission(cls, content_type=1):
        """
        启用权限系统
        :param content_type:
        :return:
        """
        cls.redis.set("permission_enable:%s" % content_type, 1)

    @classmethod
    def disable_permission(cls, content_type=1):
        """
        停用权限系统
        :param content_type:
        :return:
        """
        cls.redis.set("permission_enable:%s" % content_type, 0)

    @classmethod
    def is_enable(cls, content_type=1):
        """
        是否需要权限拦截
        """
        return cls.redis.get("permission_enable:%s" % content_type) == b'1'

    @classmethod
    def rmcache(cls, content_type=1):
        cls.remove_redis(content_type)
        cls.set_version(content_type)

    @classmethod
    def remove_redis(cls, content_type=1):
        cls.redis.delete("groups:%s" % content_type)
        cls.redis.delete("permissions:%s" % content_type)
        cls.redis.delete("company_groups:%s" % content_type)

    @classmethod
    def set_version(cls, content_type=1):
        """
        每一次修改权限，版本号为当前时间戳
        :param content_type:
        :return:
        """
        cls.redis.set("permission_version:%s" % content_type, cls.utils.now())

    @classmethod
    def get_version(cls, content_type=1):
        """
        获取当前版本号
        :param content_type:
        :return:
        """
        current_version = cls.redis.get("permission_version:%s" % content_type) or 0
        return int(current_version)

    @classmethod
    def has_update(cls, client_version, content_type=1):
        """
        客户端是否需要更新权限
        当客户端请求的header.permission_version<当前版本号时，
        设置响应header.permission_version=current_version
        客户端判断响应头里包含header.permission_version时，重新请求权限数据，并刷新客户端权限配置
        :param client_version:
        :param content_type:
        :return:
        """
        return cls.get_version(content_type) > client_version

    @classmethod
    def autoload_permission(cls, routes_hander_mp, permissions):
        """
        自动收集并创建权限条目
        该方法由进程启动时调用，请勿手动调用
        routers： self.application.handlers
        routes_hander_mp： key对应的hendler对象
        permissions: {"key": [get, post], "key1": [get, put]}
        """
        method_title = {"get": "查看", "post": "创建", "put": "修改", "delete": "删除"}
        datas = []
        for key, methods in permissions.items():
            handler = routes_hander_mp[key]
            class_docs = handler.__doc__
            for method in methods:
                method_docs = getattr(handler, method).__doc__
                # 优先选取方法描述
                if not method_docs:
                    # 其次选取类描述
                    if not class_docs:
                        name = method_title[method] + key
                        describes = "自动载入的权限"
                    else:
                        class_docs_list = class_docs.split("\n")
                        docs = class_docs_list[0] or class_docs_list[1]
                        name = method_title[method] + docs.replace(" ", "")[:10]
                        describes = class_docs.replace("\n", "").replace(" ", "")[:100]
                else:
                    method_docs_list = method_docs.split("\n")
                    docs = method_docs_list[0] or method_docs_list[1]
                    name = docs.replace(" ", "")[:10]
                    describes = method_docs.replace("\n", "").replace(" ", "")[:100]

                data = cls.create_data(**{
                    "key": key,
                    "name": name or method_title[method] + key,
                    "method": method,
                    "permission_class": "api",
                    "describes": describes,
                    "content_type": 0
                })
                datas.append(data)
        if datas:
            cls.rmcache(0)
        return datas


class GroupsModel(BaseModel, GroupsSchema):

    @classmethod
    def _parse_data(cls, doc, **kwargs):
        """
        组装、修正单条记录
        该方法若存在，框架会自动使用
        若不存在，则模式使用框架内置方法
        :param doc: 单条数据
        :param kwargs:
        :return:
        """
        if not doc:
            return {}

        if "permissions" in doc:
            doc["permissions"] = cls.utils.to_python(doc["permissions"])
        doc = cls.format_datetime(doc)

        return doc

    @classmethod
    def get_members_count(cls, company_id, *groups):
        """
        属于某商户的角色的成员数
        """
        groups_mp = {}
        members = cls.model.AdminModel.get_datas_by_spec()
        for user in members:
            if company_id and company_id != user["company_id"]:
                continue
            for gid in user["group_id"].split(","):
                groups_mp.setdefault(gid, 0)
                groups_mp[gid] += 1

        for group in groups:
            group["counts"] = groups_mp.get(str(group["id"]), 0)

        return list(groups)

    @classmethod
    def get_members(cls, company_id, groups):
        """
        属于某商户且是该角色的成员
        """
        members = []
        for user in cls.model.AdminModel.get_datas_by_spec():
            if company_id and company_id != user["company_id"]:
                continue
            if groups["id"] in list(map(int, user["group_id"].split(","))):
                members.append(user)

        return members

    @classmethod
    def get_group_list_by_page(cls, page, count, **kwargs):
        """
        列表
        :return:
        :rtype:
        """

        datas, counts = cls.get_list_by_page(page, count, **kwargs)
        return datas, counts

    @classmethod
    def create_groups(cls, name, content_type, **group):
        """
        创建角色
        name = models.CharField(max_length=128)
        state = models.IntegerField(default=1)
        describes = models.CharField(max_length=128, default="", verbose_name='描述')
        permissions = models.JSONField(default=dict)
        content_type = models.IntegerField(default=1, verbose_name='1业务角色，2管理端角色')
        assign = models.IntegerField(default=1, verbose_name='是否默认分配给商户,1是0否')
        :return:
        """

        if content_type not in [0, 1]:
            raise Exception("参数错误")
        if cls.model.GroupsModel.get_data_by_spec(sql_and={"name": name, "content_type": content_type}):
            raise Exception("已存在同名角色")
        group["name"] = name
        group["content_type"] = content_type
        cls.model.RBACPermissionModel.rmcache(content_type)
        return cls.create_data(**group)

    @classmethod
    def update_group(cls, group_id, content_type, **kwargs):
        """
        更新角色信息: 名称 描述 状态 分配
        :param id:
        :param kwargs:
        :return:
        """
        kwargs.pop("id", "")
        kwargs.pop("permissions", "")
        if kwargs:
            cls.update_data(group_id, **kwargs)
            cls.model.RBACPermissionModel.rmcache(content_type)

    @classmethod
    def update_user_group(cls, *uids, group_id=None):
        """
        修改用户的角色
        :param uids:
        :param group_id: "gid1,gid2,gid3"
        :return:
        """
        try:
            user = cls.model.AdminModel.get_admin(id=uids[0])
            company = cls.model.CompanyModel.get_company(user["company_id"])
        except Exception as e:
            return
        if not cls.get_data_by_spec(sql_and={"company_id": company["id"], "group_id": group_id}):
            return

        for uid in uids:
            cls.model.AdminModel.update_data(uid, group_id=group_id)

    @classmethod
    def update_group_permission(cls, group_id, key, method, val):
        """
        单条勾选、取消权限
        :param group_id:
        :param key:
        :param method: get or get,post 多个方法用逗号隔开
        :return:
        """
        group = cls.get_data_by_kv(id=group_id)
        if group:
            permissions = group["permissions"]
            if val:
                permissions[key + ":" + method] = val
            else:
                permissions.pop(key + ":" + method, "")

            group.pop("group_id", "")
            cls.update_data(group_id, **group)
            cls.model.RBACPermissionModel.rmcache(group["content_type"])

    @classmethod
    def update_group_permissions(cls, group_id, permissions):
        """
        批量勾选、取消权限
        :param group_id:
        :param permissions:
            {
                "key:get": 1,
                "key:get,post": 1
            }
        :return:
        """
        group = cls.get_data_by_kv(id=group_id)
        if group:
            cls.update_data(group_id, permissions=cls.utils.to_json(permissions))
            cls.model.RBACPermissionModel.rmcache(group["content_type"])

    @classmethod
    def get_groups_list(cls, content_type=1):
        """
        前端
        返回角色（含权限）
        :return:
        [
            {
                "id": "id",
                "name": "角色名称",
                # "group_class": "技术部",             # 预留，自定义标签
                "state": 1,                           # 1可以 0 禁用 默认1
                "content_type": 1 or 0,             # 1 业务端使用，0系统管理端
                "assign": 0,                        # 是否默认分配给商户 1是0否
                "permissions": {
                    "control2-xxx-api:get": 1,        # 1勾选，0未勾选
                    "control2-xxx-api2:get,post,put,delete": 1,  # 组合方法
                    "monitoring-tasks-api:post": 1
                }
            },
            ...
        ]
        """
        groups = cls.redis.get("groups:%s" % content_type)
        if groups:
            groups = cls.utils.to_python(groups)
        else:
            groups = []
            for g in cls.get_datas_by_spec(sql_and=dict(state=1, content_type=content_type)):
                groups.append(g)
            if groups:
                cls.redis.set("groups:%s" % content_type, cls.utils.to_json(groups))
        return groups

    @classmethod
    def get_groups_mapping(cls, content_type=1):
        """
        获取角色拥有的权限，多个方法组合为一个权限时，会得到多条k：v
        :return:
        {
            1: {
                "...": **groups,                  # 角色基础信息
                "permissions": {
                    groups.key:get: 1,         # 键：权限key:方法(单个方法)
                    groups.key:post: 1,        # 键：权限key:方法(单个方法)
                    ...
                }
            }
        }
        """
        groups = {str(g["id"]): g for g in cls.get_groups_list(content_type)}
        for id, group in groups.items():
            temp = {}
            permissions = group.pop("permissions")
            # get,post 多个方法平铺
            for keys, val in permissions.items():
                if not val:
                    continue  # 0代表没有权限，跳过
                key, methods = keys.split(":")
                if "," in methods:
                    for method in methods.split(","):
                        temp[key + ":" + method] = val
                else:
                    temp[keys] = val
            group["permissions"] = temp
        return groups

    @classmethod
    def get_system_groups(cls):
        """
        获取系统角色
        :return: [1, 2, 3]
        """
        return cls.model.CompanyGroupsModel.get_company_groups(0)


class CompanyGroupsModel(BaseModel, CompanyGroupsSchema):

    @classmethod
    def _parse_data(cls, doc, **kwargs):
        """
        组装、修正单条记录
        该方法若存在，框架会自动使用
        若不存在，则模式使用框架内置方法
        :param doc: 单条数据
        :param kwargs:
        :return:
        """
        if not doc:
            return {}

        doc = cls.format_datetime(doc)

        return doc

    @classmethod
    def update_company_group(cls, company_id, group_ids):
        """
        修改、新增商户的角色
        :param old_group:
        :param new_group:
        :return:
        """
        groups_mp = cls.get_company_groups(1)
        groups_mp.update({"1": cls.get_system_groups()})
        old_group_id = groups_mp.get(str(company_id), [])
        group_ids = [group_ids] if not isinstance(group_ids, (list, tuple)) else group_ids
        diff_group_id = set(old_group_id) - set(group_ids)
        for group_id in group_ids:
            data = cls.get_data_by_spec(sql_and=dict(company_id=company_id, group_id=int(group_id)))
            if not data:
                cls.create_data(company_id=company_id, group_id=group_id)
            else:
                cls.update_data(data["id"], company_id=company_id, group_id=group_id)

        # 其他的角色，移除
        if diff_group_id:
            cls.remove_company_group(company_id, list(diff_group_id))
        cls.model.RBACPermissionModel.rmcache(1)

    @classmethod
    def remove_company_group(cls, company_id, group_ids):
        """
        移除商户的某个角色：所有已分配该角色的用户，将被解除角色绑定
        :param company_id:
        :param group_id:
        :return:
        """
        group_ids = [str(i) for i in ([group_ids] if not isinstance(group_ids, (list, tuple)) else group_ids)]
        # 删除角色
        for group_id in group_ids:
            cls.mysql.execute(f"delete from {cls.collection} where company_id=%s and group_id=%s",
                              [company_id, int(group_id)])

        # 从该用户角色中剔除这些角色
        for user in cls.model.AdminModel.get_datas_by_spec(sql_and=dict(company_id=company_id)):
            user_group_ids = set(user["group_id"].split(","))
            group_ids = set(group_ids)
            user_group_ids.difference_update(group_ids)
            user["group_id"] = ",".join(list(user_group_ids))
            cls.model.AdminModel.update_data(user["id"], group_id=user["group_id"])

        cls.model.RBACPermissionModel.rmcache(1)

    @classmethod
    def get_system_groups(cls):
        """
        获取系统角色
        :return: [1, 2, 3]
        """
        return cls.get_company_groups(0)

    @classmethod
    def get_company_groups(cls, content_type=1):
        """
        商户对应的角色
        :param content_type:
        :return:["2": [1, 2, 3], "3": [4, 5, 6]]
        """
        company_groups = {}
        cache_datas = cls.redis.get("company_groups:%s" % content_type)
        if cache_datas:
            company_groups = cls.utils.to_python(cache_datas)
        else:
            if content_type == 1:
                datas = cls.get_datas_by_spec()
                for item in datas:
                    if str(item["company_id"]) not in company_groups:
                        company_groups[str(item["company_id"])] = []
                    company_groups[str(item["company_id"])].append(item["group_id"])
            else:
                company_groups = []
                for group in cls.model.GroupsModel.get_datas_by_spec(sql_and=dict(content_type=content_type)):
                    company_groups.append(group["id"])

            if company_groups:
                cls.redis.set("company_groups:%s" % content_type, cls.utils.to_json(company_groups))

        return company_groups